
/* Initialization */

// Initializer: block of code that's executed right after object is created
// Class, struct and enum have initializers.

class Vehicle {
   var currentSpeed:Int = 2
   let name:String
   var description:String
   var color:String?      // Optional property doesn't need to be initialized
   init() {
      //currentSpeed = 1  // Better put in the default value
      name = "Unknown"   // constant can only be modified here, it cannot be modified by subclass's init()
      //name = ""  // constant cannot be set multiple times
      description = "\(name) running at speed of \(currentSpeed)"
   }

   init(speed:Int, name:String) {
      currentSpeed = speed
      self.name = name // note self is required here
      description = "\(name) running at speed of \(currentSpeed)"
   }
}
let v1 = Vehicle()
let v2 = Vehicle(speed:23, name: "Car")  // initializer provide default external name for EVERY parameter

// Arguments has default external parameters:
// 1. Function parameters that have default values
// 2. All method parameters except the first parameter
// 3. All initializer parameters












// Auto Generated Initializers
// 1. Default Initializer - class and struct
class Dog {           // Condition for default initializer:
   var age = 1        // 1: all properties are initialized
   var name = ""      // 2: no init function being defined
}

let d = Dog()  // default initializer

// 2. Memberwise Initializer - struct
struct Cat {
   var age:Int = 1
   var name = ""      // Condition for memberwise initializer: 
                      //     No init function being defined
}
let c1 = Cat(age: 2, name: "Will")
let c2 = Cat()    // This won't compile if any property is not initialized















// Designated Initializer and Convenience Initializer (only for class)
class Travaller {
   var type:String = ""
   init(type:String) {
      self.type = type
   }
}

class Vehicle2 : Travaller {
   var currentSpeed:Int
   let name:String
   var description:String
   
   init(speed:Int, name:String) {   // Designated Initializer
      currentSpeed = speed
      self.name = name // note self is required here
      description = "\(name) running at speed of \(currentSpeed)"
      super.init(type: "Vehicle")
   }
   
   convenience init() {
      self.init(speed:0, name:"")   // Calling delegate initializer
   }
}
// Rules:
//  1. Every class must have one designated initializer: self defined, inherited, or auto generated
//  2. Designated Initializer must fully initialize all properties and calling immediate superclass's desginated initializer
//  3. Convenience initializer must call another initilaizer in same class, and ultimately must call a designated initializer.












// Ordering Rules:
class Vehicle3 : Travaller {
   var currentSpeed:Int
   let name:String
   var description:String

   init(speed:Int, name:String) { // Designated Initializer
      // Step 1: Initialize its own properties
      currentSpeed = speed
      self.name = name
      description = "\(name) running at speed of \(currentSpeed)"
      // Step 2: Calling superclass initializer
      super.init(type: "Vehicle")
      // Step 3: overwrite superclass's properties and/or its own properties
      type = "Not really Vehicle"
   }
   
   convenience init() {
      // Step 1: calling another initializer
      self.init(speed:0, name:"")
      // Step 2: overwriting properties
      description = "unique description"
      type = "another type"
   }
   
}

// Be careful of doing following in initializers:
// 1. calling methods
// 2. accessing properties
// because in theory the object is not fully initilized before init() finishes.










/* Deinitialization */
class Vehicle4 : Travaller {
   deinit {
      // release resources.
   }
}
// 1. deinit function cannot be called explicitly, e.g. v1.deinit()
// 2. Superclass's deinit is automatically called at the end of subclass's deinit
// 3. Free to use methods and properties because object is only dealloced after deinit

enum Direction {
   case East, West, South, North
   init() {
      self = Direction.North   // self must be assigned
   }
}
// enum has no auto-generated initializer
var d1 = Direction() // Error if init() is not defined


// struct and enum have delegate initializers
enum Direction2 {
   case East, West, South, North
   init() {
      self.init(flag: 1)
   }
   init(flag:Int) {
      self = Direction2.North
   }
}










