

/* Type Casting */

// It only happens to related classes: 
//      downcasting -- cast from parent class to child class
//      upcasting   -- cast from child class to parent class

class Media {
   var name:String = ""
   init(name:String) {
      self.name = name
   }
}

class Song:Media {}

class Movie:Media {}


let s1 = Song(name: "Fireproof")
var m1:Media = s1   // upcasting can be implicit

//var s2:Song = m1
var s2:Song = m1 as! Song  // downcasting must be explicit and correct
//var x1:Movie = m1 as Movie  // Incorect casting results in runtime error











// Another example
let a = [Song(name: "Fireproof"), Movie(name: "Shrek"), Media(name: "King Kong")]
a   // a is an array of Media

// a[0] is a Media in the eyes of compiler, but it really is a Song
var s3:Song = a[0] as! Song   // downcasting, it must be explicit

//var m2:Song = a[1] as Song  // Runtime Error: a[1] cannot be casted into a Song
//var m3:Song = a[2] as Song  // same error


// What if I forgot which object is what?
if a[1] is Song {
   let m2:Song = a[1] as! Song
}

// Optional Casting
let m2 = a[1] as? Song  // returns nil if a[1] is not a Song. No runtime error

// Optional Binding
if let m4 = a[1] as? Song {
   print("a[1] is really a Song: \(m4.name)")
}

// Note: 
//  Casting doesn't modify the object, it merely changes compiler's view of it.












// Build-in data types are made of structs
let x  = 45
//let y = x as Double   // Not work
// Reasons: They don't belong to same inheritance hierachy.



























// AnyObject

class Dog {}

var a1:[AnyObject] = [Media(name: "King Kong"), Song(name: "Fireproof"), Dog()]
// a1 can store any object data (not struct, enum, or closure)
for x in a1 {
   if x is Dog {
      print("A Dog is found")
   }
}

let a2:[AnyObject] = [Media(name: "King Kong"), Song(name: "Fireproof")]
let am = a2 as! [Media]  // An array of Media's. Can't work on a1


// Any

var a3 = [Any]()  // a3 can contain anything
a3.append(2)
a3.append(2.0)
a3.append("Bob")
a3.append(Dog())
a3.append((2, "John"))
let p:Int? = 2
a3.append(p)

enum Direction {
    case EAST, WEST, SOUTH, NORTH
}
a3.append(Direction.EAST) // Store a enum

var c2 = {    //  A Closure
    (a:Int, b:Int)-> Bool in
    return a < b
}
a3.append(c2)  // Store a closure

for x in a3 {
   switch x {
   case 2 as Int:     print("it is Int 2")
   case 2 as Double:  print("it is Double 2")
   case let d as Dog: print("it is Dog")
   default:           print("Others")
   }
}



// AnyObject and Any should not be abused.



















