


/*
 * 1. Overflow Operators
 */

var x = Int.max

//x = x + 1   // Overflow is trapped as an error

x = x &+ 1  // No error, the result is Int.min

Int.min &- 1
Int.min &* 2

// overflow divide
//5 &/ 0  // returns 0

// overflow modulo
//5 &% 0  // returns 0





/*
* 2. Bitwise Operators
*/

var y:UInt8 = 0b00111000   // y is a 8-bit unsigned integer
~y                // NOT     -> 0b11000111
y & 0b11110000    // AND     -> 0b00001000
y | 0b11110000    // OR      -> 0b00111000
y ^ 0b11110000    // XOR (exclusive or)
y << 1            // Logic shift (Zeros are filled)
y >> 4            // bits moved beyond boundary are discarded

var z:Int8 = -1   // z is 8-bit signed integer
z << 1
z >> 1    // first bit is sign bit. When shift right, empty bits are filled with sign bit

 // y is an unsigned int, so it doesn't have sign bit




/*
* 3. Operator Overload
*/
struct Vector {
    var x = 0
    var y = 0
//    func == (lhs: Self, rhs: Self) -> Bool {
//        return true
//    }
}


func + (left:Vector, right:Vector) -> Vector {
    let newX = left.x + right.x
    let newY = left.y + right.y
    return Vector(x:newX, y:newY)
}
Vector(x: 2, y: 3) + Vector(x: 4, y: 5)


// Prefix Operator
prefix func - (v:Vector) -> Vector {
    return Vector(x: -v.x, y: -v.y)
}
-Vector(x: 2, y: 3)


// Postfix Operator
postfix func ++( v:inout Vector) -> Vector {
    let tmp = v
    v = Vector(x: v.x+1, y: v.y+1)
    return tmp
}
var v1 = Vector(x: 2, y: 3)
v1++
v1

// Other operator you can overload: +=, -=, *=, /=, ==, !=
// These are standard operators












// Custom Operator
infix operator +++ {}  // must be marked as infix, or prefix, or postfix
// custom operator must be declared first.


func +++ (left:Vector, right:Vector) -> Vector {
    let newX = left.x + right.x
    let newY = left.y + right.y
    return Vector(x:newX, y:newY)
}





